/*
ID: espr1t
TASK: Ragain
KEYWORDS: Testgen
*/

#include <cstdio>
#include <random>
#include <vector>
#include <algorithm>
#include <ctime>
#include <cassert>

#include "RagainSolution.h"

using namespace std;

mt19937 mt;
int rand() {
    int ret = mt();
    return ret < 0 ? -ret : ret;
}

struct Test {
    int n;
    int l, w, h;
    vector <Point> points;
};
vector <Test> tests;

void sampleInput() {
    Test test;
    test.n = 14, test.l = 10, test.w = 5, test.h = 4;
    int points[14][2] = {
        {3, 4},
        {0, 2},
        {5, 1},
        {10, 10},
        {4, 0},
        {8, 7},
        {2, 7},
        {6, 5},
        {9, 2},
        {7, 3},
        {5, 8},
        {6, 5},
        {4, 2},
        {3, 6}
    };
    for (int i = 0; i < test.n; i++)
        test.points.push_back(Point(points[i][0], points[i][1]));
    tests.push_back(test);
}

void singlePoint() {
    Test test;
    test.n = 1, test.l = 2, test.w = 2, test.h = 2;
    test.points.push_back(Point(1, 1));
    tests.push_back(test);
}

void neverCovered() {
    Test test;
    test.n = 36, test.l = 5, test.w = 1, test.h = 1;
    for (int x = 0; x <= test.l; x++)
        for (int y = 0; y <= test.l; y++)
            test.points.push_back(Point(x, y));
    random_shuffle(test.points.begin(), test.points.end());
    tests.push_back(test);
}

void randomTest(int n_, int l_, bool randomized = false) {
    fprintf(stderr, "Generating random test with n = %d...\n", n_);

    mt.seed(l_);

    Test test;
    test.n = n_, test.l = l_;
    for (int i = 0; i < test.n; i++)
        test.points.push_back(Point(rand() % (test.l + 1), rand() % (test.l + 1)));
    
    if (randomized)
        random_shuffle(test.points.begin(), test.points.end());
    
    test.w = test.h = 1;
    int best = -1;
    int leftW = 1, rightW = test.l;
    while (leftW <= rightW) {
        int midW = (leftW + rightW) / 2;
        
        int bestH = 0;
        int leftH = 1, rightH = test.l;
        while (leftH <= rightH) {
            int midH = (leftH + rightH) / 2;
            int cur = solve(test.n, test.l, midW, midH, test.points);
            if (cur >= 50 * test.n / 100)
                leftH = midH + 1, bestH = midH;
            else
                rightH = midH - 1;
        }
        
        int cur = solve(test.n, test.l, midW, bestH, test.points);
        if (best < cur)
            best = cur, test.w = midW, test.h = bestH;

        if ((long long)test.w * test.h < (long long)midW * bestH)
            leftW = midW + 1;
        else
            rightW = midW - 1;
    }
    tests.push_back(test);
}

void bigX(int n_, int l_, bool randomized = false) {
    fprintf(stderr, "Generating big X test with n = %d...\n", n_);
    
    Test test;
    test.n = n_, test.l = l_;
    
    assert(test.n % 2 == 0);
    int off = test.l / (test.n / 2);
    assert(off > 0);

    for (int i = 0; i < test.n / 2; i++)
        test.points.push_back(Point(i * off, i * off));
    for (int i = 0; i < test.n / 2; i++)
        test.points.push_back(Point(i * off, test.l - i * off));
    test.w = test.h = test.l / 2 - 1;
    
    if (randomized) {
        random_shuffle(test.points.begin(), test.points.end());
    }
    tests.push_back(test);
}

int mW, mH;
void daq(vector<Point>& points, int rem, int sX, int sY, int eX, int eY) {
    if (rem == 0) {
        if ((long long)mW * mH < (long long)(eX - sX + 1) * (eY - sY + 1))
            mW = eX - sX + 1, mH = eY - sY + 1;
        return;
    }
    points.push_back(Point((sX + eX) / 2, (sY + eY) / 2));
    rem--;
    daq(points, rem / 4 + (rem % 4 > 0), sX, sY, (sX + eX) / 2 - 1, (sY + eY) / 2 - 1);
    daq(points, rem / 4 + (rem % 4 > 1), (sX + eX) / 2 + 1, sY, eX, (sY + eY) / 2 - 1);
    daq(points, rem / 4 + (rem % 4 > 2), sX, (sY + eY) / 2 + 1, (sX + eX) / 2 - 1, eY);
    daq(points, rem / 4 + (rem % 4 > 3), (sX + eX) / 2 + 1, (sY + eY) / 2 + 1, eX, eY);
}

void divideAndConquer(int n_, int l_, bool randomized = false) {
    fprintf(stderr, "Generating Div&Conq test with n = %d...\n", n_);
    
    Test test;
    test.n = n_, test.l = l_;
    mW = mH = 0;
    daq(test.points, test.n, 0, 0, test.l, test.l);
    test.w = mW * 4 + 1, test.h = mH * 4 + 1;

    if (randomized) {
        random_shuffle(test.points.begin(), test.points.end());
    }
    tests.push_back(test);
}

void inscribedRectangles(int n_, int l_, bool randomized = false) {
    fprintf(stderr, "Generating inscribed rectangles test with n = %d...\n", n_);

    Test test;
    test.n = n_ - n_ % 8, test.l = l_;
    
    int numRects = test.n / 8;
    int offX = max(1, (test.l - test.l / 4) / (numRects * 2));
    int offY = max(1, (test.l - test.l / 5) / (numRects * 2));
    int sX = 0, sY = 0, eX = test.l, eY = test.l;
    for (int i = 0; i < numRects; i++) {
        test.points.push_back(Point(sX, sY));
        test.points.push_back(Point(sX, eY));
        test.points.push_back(Point(eX, eY));
        test.points.push_back(Point(eX, sY));
        test.points.push_back(Point(sX, (sY + eY) / 2));
        test.points.push_back(Point((sX + eX) / 2, eY));
        test.points.push_back(Point(eX, (sY + eY) / 2));
        test.points.push_back(Point((sX + eX) / 2, sY));
        sX += offX, sY += offY, eX -= offX, eY -= offY;
    }
    test.w = eX - sX, test.h = eY - sY;
    test.w += 2 * offX, test.h += 2 * offY;
    
    while (solve(test.n, test.l, test.w, test.h, test.points) == -1) {
        test.w = test.w * 3 / 2; test.h = test.h * 3 / 2;
    }
    
    if (randomized) {
        random_shuffle(test.points.begin(), test.points.end());
    }
    tests.push_back(test);
}

void decreasingLines(int n_, int l_, bool randomized = false) {
    fprintf(stderr, "Generating decreasing lines test with n = %d...\n", n_);
    
    Test test;
    test.n = n_, test.l = l_;

    int numLines = 5 + rand() % 6;
    if (numLines < test.n / test.l) {
        numLines = test.n / test.l + !!(test.n % test.l);
    }
    
    for (int i = 0; i < numLines; i++) {
        int inLine = test.n / numLines + (test.n % numLines > i);
        int sY = rand() % (test.l + 1);
        for (int c = 0; c < inLine; c++) {
            int x = rand() % (test.l + 1);
            int y = (sY + x) % (test.l + 1);
            test.points.push_back(Point(x, y));
        }
    }
    
    int inc = max(1, test.l / 100);
    test.w = test.h = 1;
    while (solve(test.n, test.l, test.w, test.h, test.points) == -1) {
        if (rand() % 2)
            test.w += inc;
        else
            test.h += inc * 7 / 5;
    }
    test.w = min(test.w, test.l);
    test.h = min(test.h, test.l);

    if (randomized) {
        random_shuffle(test.points.begin(), test.points.end());
    }
    tests.push_back(test);
}

void sanityCheck(int t) {
    Test& test = tests[t];
    if (test.n < 1 || test.n > 100000) {
        fprintf(stderr, "ERROR: Test %d has invalid N (%d).\n", t, test.n);
        exit(0);
    }
    if (test.l < 1 || test.l > 1000000000) {
        fprintf(stderr, "ERROR: Test %d has invalid L (%d).\n", t, test.l);
        exit(0);
    }
    
    if (test.w < 1 || test.w > test.l) {
        fprintf(stderr, "ERROR: Test %d has invalid W (%d).\n", t, test.w);
        exit(0);
    }
    
    if (test.h < 1 || test.h > test.l) {
        fprintf(stderr, "ERROR: Test %d has invalid H (%d).\n", t, test.h);
        exit(0);
    }
    
    if (test.n != (int)test.points.size()) {
        fprintf(stderr, "ERROR: Test %d has incomplete set of poitns (N is %d vs %d).\n",
            t, test.n, (int)test.points.size());
        exit(0);
    }
    
    for (int i = 0; i < test.n; i++) {
        if (test.points[i].x < 0 || test.points[i].x > test.l) {
            fprintf(stderr, "ERROR: Test %d has invalid X on point %d (%d).\n",
                t, i, test.points[i].x);
            exit(0);
        }
        if (test.points[i].y < 0 || test.points[i].y > test.l) {
            fprintf(stderr, "ERROR: Test %d has invalid Y on point %d (%d).\n",
                t, i, test.points[i].y);
            exit(0);
        }
    }
}

void printTests() {
    for (int t = 0; t < (int)tests.size(); t++) {
        fprintf(stderr, "Priting testcase %d...\n", t);
        sanityCheck(t);

        unsigned sTime = clock();
        int ans = solve(tests[t].n, tests[t].l, tests[t].w, tests[t].h, tests[t].points);
        fprintf(stderr, "  >> answer is %d (out of %d).\n", ans, tests[t].n);
        fprintf(stderr, "  >> execution time: %.3lfs\n", (double)(clock() - sTime) / CLOCKS_PER_SEC);

        char inpFile[32], solFile[32];
        sprintf(inpFile, "Ragain.%02d.in", t);
        sprintf(solFile, "Ragain.%02d.sol", t);

        FILE* inp = fopen(inpFile, "wt");
        fprintf(inp, "%d %d %d %d\n", tests[t].n, tests[t].l, tests[t].w, tests[t].h);
        for (int i = 0; i < (int)tests[t].points.size(); i++)
            fprintf(inp, "%d %d\n", tests[t].points[i].x, tests[t].points[i].y);
        fclose(inp);

        FILE* sol = fopen(solFile, "wt");
        fprintf(sol, "%d\n", ans);
        fclose(sol);
    }
}

int main(void) {
    mt.seed(42);
    sampleInput();
    
    // [1, 500], small coordinates
    singlePoint();
    neverCovered();
    randomTest(20, 20);
    bigX(100, 50);
    divideAndConquer(128, 64);
    inscribedRectangles(200, 200);
    randomTest(200, 500);
    
    // [1000, 2000]
    randomTest(1000, 1000000);
    bigX(2000, 1000000000);
    divideAndConquer(1337, 536870912);
    inscribedRectangles(1664, 10000000);
    decreasingLines(2000, 1000000);
    
    // [10000, 20000]
    randomTest(10000, 1000000);
    bigX(20000, 1000000000, true);
    divideAndConquer(12345, 1234567, true);
    inscribedRectangles(19376, 1000000000, true);
    decreasingLines(20000, 1000000000, true);
    
    // [90000, 100000]
    randomTest(90000, 1000000);
    bigX(100000, 1000000000);
    divideAndConquer(99999, 999999);
    inscribedRectangles(100000, 1000000000);
    decreasingLines(100000, 1000000);
    
    // Large randoms
    randomTest(100000, 1000000000 - 2);
    randomTest(100000, 1000000000 - 1);
    randomTest(100000, 1000000000 - 0);
    
    printTests();
    return 0;
}
