/*
ID: espr1t
TASK: Ragain
KEYWORDS: Hard, Data Structures, Sweepline
*/

#include <cstdio>
#include <queue>
#include <map>
#include <algorithm>
#include <random>
#include <ctime>

using namespace std;
FILE *in = stdin, *out = stdout;

const int MAX = 100004;

mt19937 mt;
int rand() {
    int num = mt();
    return num >= 0 ? num : -num;
}

struct Point {
    int x, y;
    bool operator < (const Point& r) const {
        return x != r.x ? x > r.x : y > r.y;
    }
};

int n;
int l, w, h;
Point points[MAX];

int atLeastH;
map <int, int> yCoords;
priority_queue <Point> add, rem;

void updateCoord(int y, int value) {
    yCoords[y] += value;
    map <int, int> :: iterator it = yCoords.find(y);
    if (it->second != 0 && it->second - value != 0)
        return;
    int lower, upper;
    it--; lower = y - it->first; it++;
    it++; upper = it->first - y; it--;
    atLeastH += ((lower >= h) + (upper >= h) - (lower + upper >= h)) * value;
    if (it->second == 0)
        yCoords.erase(it);
}


void updateLine(int& xPosition) {
    bool updated = true;
    while (updated) {
        updated = false;
        
        // Add new points
        while (!add.empty() && add.top().x - w < xPosition) {
            Point point = add.top(); add.pop();
            updateCoord(point.y, +1);
        }

        // Move the sweep line and remove points that are already behind
        while (!rem.empty() && (rem.top().x <= xPosition || atLeastH == 0)) {
            Point point = rem.top(); rem.pop();
            if (atLeastH == 0)
                xPosition = point.x, updated = true;
            updateCoord(point.y, -1);
        }
    }
}

int solveLog() {
    atLeastH = 0;
    yCoords.clear();
    while (!add.empty()) add.pop();
    while (!rem.empty()) rem.pop();
    
    yCoords[0] = 1, yCoords[l] = 1, atLeastH = (l >= h);
    
    int xPosition = 0;
    for (int i = 0; i < n; i++) {
        // Boundaries don't change anything
        if (points[i].x == 0 || points[i].x == l || points[i].y == 0 || points[i].y == l)
            continue;

        // Unnecessary (already passed)
        if (points[i].x <= xPosition)
            continue;

        // Add for future evaluation
        add.push(points[i]), rem.push(points[i]);

        // Move the sweep line if possible
        updateLine(xPosition);
        if (xPosition + w > l)
            return i + 1;
    }
    return -1;
}

bool covered(int size) {
    static Point cur[MAX];
    memcpy(cur, points, size * sizeof(points[0]));
    sort(cur, cur + size);
    reverse(cur, cur + size);
    
    atLeastH = (l >= h);
    yCoords.clear(); yCoords[0] = 1, yCoords[l] = 1;
    
    int xPosition = 0, idx = 0;
    for (int i = 0; i < size; i++) {
        if (cur[i].x - w >= xPosition)
            return false;
        
        updateCoord(cur[i].y, +1);
        while (atLeastH == 0) {
            xPosition = cur[idx].x;
            while (idx <= i && cur[idx].x <= xPosition)
                updateCoord(cur[idx++].y, -1);
        }
        if (xPosition + w > l)
            return true;
    }
    return false;
}

int solveLogLog() {
    if (!covered(n)) {
        return -1;
    }
    int left = 1, right = n;
    while (left <= right) {
        int mid = (left + right) / 2;
        if (covered(mid))
            right = mid - 1;
        else
            left = mid + 1;
    }
    return right + 1;
}

int slow() {
    vector <long long> x, y;
    x.push_back(w), y.push_back(h);
    
    for (int i = 0; i < n; i++) {
        if (points[i].x + w <= l)
            x.push_back(points[i].x * 2LL + w);
        if (points[i].y + h <= l)
            y.push_back(points[i].y * 2LL + h);
    }
    sort(x.begin(), x.end());
    x.resize(unique(x.begin(), x.end()) - x.begin());
    sort(y.begin(), y.end());
    y.resize(unique(y.begin(), y.end()) - y.begin());
    
    if ((long long)n * x.size() * y.size() > 10000000000LL)
        return -42;
    
    vector < pair <long long, long long> > candid;
    for (int i = 0; i < (int)x.size(); i++)
        for (int c = 0; c < (int)y.size(); c++)
            candid.push_back(make_pair(x[i], y[c]));
    
    for (int i = 0; i < n; i++) {
        int idx = 0;
        for (int c = 0; c < (int)candid.size(); c++) {
            if (points[i].x * 2LL - w >= candid[c].first ||
                points[i].x * 2LL + w <= candid[c].first ||
                points[i].y * 2LL - h >= candid[c].second ||
                points[i].y * 2LL + h <= candid[c].second) {
                    candid[idx++] = candid[c];
            }
        }
        candid.resize(idx);
        if (candid.empty())
            return i + 1;
    }
    return -1;
}

void test() {
    mt.seed(42);
    int NUM_ITERS = 1000;
    for (int iter = 1; iter <= NUM_ITERS; iter++) {
        fprintf(stderr, "At iteration %d...\n", iter);
        n = 500 + rand() % 501;
        l = 10000 + rand() % 10011;
        w = rand() % l + 1, h = rand() % l + 1;
        for (int i = 0; i < n; i++) {
            points[i].x = rand() % l + 1;
            points[i].y = rand() % l + 1;
        }
        int ans1 = slow();
        int ans3 = solveLogLog();
        int ans2 = solveLog();

        fprintf(stderr, "  >> %d vs %d vs %d (n is %d)\n", ans1, ans2, ans3, n);
        if (ans1 != ans2 || ans1 != ans3) {
            fprintf(stderr, "Different answers! Test was:\n");
            fprintf(stderr, "%d %d %d %d\n", n, l, w, h);
            for (int i = 0; i < n; i++)
                fprintf(stderr, "%d %d\n", points[i].x, points[i].y);
            break;
        }
    }
    fprintf(stderr, "Finished all %d iterations!\n", NUM_ITERS);
}

int main(void) {
    // test()
    
    int numTests = 25;
    for (int test = 0; test <= numTests; test++) {
        fprintf(stderr, "Currently solving testcase %d...\n", test);

        char inpFile[32];
        sprintf(inpFile, "Ragain.%02d.in", test);
        FILE* in = fopen(inpFile, "rt");
        fscanf(in, "%d %d %d %d", &n, &l, &w, &h);
        for (int i = 0; i < n; i++)
            fscanf(in, "%d %d", &points[i].x, &points[i].y);
        fclose(in);
        
        unsigned sTime1 = clock();
        int ans1 = slow();
        fprintf(stderr, "  >> ans1 = %d (execution time %.3lfs)\n",
            ans1, (double)(clock() - sTime1) / CLOCKS_PER_SEC);

        unsigned sTime2 = clock();
        int ans2 = solveLogLog();
        fprintf(stderr, "  >> ans2 = %d (execution time %.3lfs)\n",
            ans2, (double)(clock() - sTime2) / CLOCKS_PER_SEC);
        
        unsigned sTime3 = clock();
        int ans3 = solveLog();
        fprintf(stderr, "  >> ans3 = %d (execution time %.3lfs)\n",
            ans3, (double)(clock() - sTime3) / CLOCKS_PER_SEC);
    }
    return 0;
}
